@@ -1,4 +1,4 @@
-/**
+/*
  * Copyright (C) 2014 Google Inc.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
@@ -39,20 +39,18 @@ import com.google.inject.spi.ProvisionListenerBinding;
 import com.google.inject.spi.ScopeBinding;
 import com.google.inject.spi.TypeConverterBinding;
 import com.google.inject.spi.TypeListenerBinding;
-
-import junit.framework.TestCase;
-
 import java.lang.annotation.Annotation;
 import java.lang.ref.WeakReference;
 import java.util.List;
 import java.util.Map;
 import java.util.Set;
+import junit.framework.TestCase;
 
 /**
  * Tests for {@link WeakKeySet}.
- * <p>
- * Multibinding specific tests can be found in MultibinderTest and MapBinderTest.
- * 
+ *
+ * <p>Multibinding specific tests can be found in MultibinderTest and MapBinderTest.
+ *
  * @author dweis@google.com (Daniel Weis)
  */
 public class WeakKeySetTest extends TestCase {
@@ -68,8 +66,8 @@ public class WeakKeySetTest extends TestCase {
     TestState state = new TestState();
     Key<Integer> key = Key.get(Integer.class);
     Object source = new Object();
-    
-    WeakReference<Key<Integer>> weakKeyRef = new WeakReference<Key<Integer>>(key);
+
+    WeakReference<Key<Integer>> weakKeyRef = new WeakReference<>(key);
 
     set.add(key, state, source);
     assertInSet(set, key, 1, source);
@@ -84,13 +82,13 @@ public class WeakKeySetTest extends TestCase {
     key = null;
     awaitClear(weakKeyRef);
   }
-  
+
   public void testEviction_nullSource() {
     TestState state = new TestState();
     Key<Integer> key = Key.get(Integer.class);
     Object source = null;
-    
-    WeakReference<Key<Integer>> weakKeyRef = new WeakReference<Key<Integer>>(key);
+
+    WeakReference<Key<Integer>> weakKeyRef = new WeakReference<>(key);
 
     set.add(key, state, source);
     assertInSet(set, key, 1, source);
@@ -120,10 +118,10 @@ public class WeakKeySetTest extends TestCase {
     set.add(key2, state2, source2);
     assertInSet(set, key2, 2, source1, source2);
 
-    WeakReference<Key<Integer>> weakKey1Ref = new WeakReference<Key<Integer>>(key1);
-    WeakReference<Key<Integer>> weakKey2Ref = new WeakReference<Key<Integer>>(key2);
-    WeakReference<Object> weakSource1Ref = new WeakReference<Object>(source1);
-    WeakReference<Object> weakSource2Ref = new WeakReference<Object>(source2);
+    WeakReference<Key<Integer>> weakKey1Ref = new WeakReference<>(key1);
+    WeakReference<Key<Integer>> weakKey2Ref = new WeakReference<>(key2);
+    WeakReference<Object> weakSource1Ref = new WeakReference<>(source1);
+    WeakReference<Object> weakSource2Ref = new WeakReference<>(source2);
 
     Key<Integer> key = key1 = key2 = Key.get(Integer.class);
     state1 = null;
@@ -134,11 +132,11 @@ public class WeakKeySetTest extends TestCase {
     assertInSet(set, key, 1, source2);
 
     source1 = source2 = null;
-    
+
     awaitClear(weakSource1Ref);
     // Key1 will be referenced as the key in the sources backingSet and won't be
     // GC'd.
-    
+
     // Should not be GC'd until state2 goes away.
     assertNotNull(weakSource2Ref.get());
 
@@ -153,7 +151,7 @@ public class WeakKeySetTest extends TestCase {
     // Now that the backing set is emptied, key1 is released.
     awaitClear(weakKey1Ref);
   }
-  
+
   public void testNoEviction_keyOverlap_2x() {
     TestState state1 = new TestState();
     TestState state2 = new TestState();
@@ -168,8 +166,8 @@ public class WeakKeySetTest extends TestCase {
     set.add(key2, state2, source2);
     assertInSet(set, key2, 2, source1, source2);
 
-    WeakReference<Key<Integer>> weakKey1Ref = new WeakReference<Key<Integer>>(key1);
-    WeakReference<Key<Integer>> weakKey2Ref = new WeakReference<Key<Integer>>(key2);
+    WeakReference<Key<Integer>> weakKey1Ref = new WeakReference<>(key1);
+    WeakReference<Key<Integer>> weakKey2Ref = new WeakReference<>(key2);
 
     Key<Integer> key = key1 = key2 = Key.get(Integer.class);
 
@@ -197,9 +195,9 @@ public class WeakKeySetTest extends TestCase {
     assertInSet(set, key2, 1, source);
     assertInSet(set, key1, 1, source);
 
-    WeakReference<Key<Integer>> weakKey1Ref = new WeakReference<Key<Integer>>(key1);
-    WeakReference<Key<Integer>> weakKey2Ref = new WeakReference<Key<Integer>>(key2);
-    WeakReference<Object> weakSourceRef = new WeakReference<Object>(source);
+    WeakReference<Key<Integer>> weakKey1Ref = new WeakReference<>(key1);
+    WeakReference<Key<Integer>> weakKey2Ref = new WeakReference<>(key2);
+    WeakReference<Object> weakSourceRef = new WeakReference<>(source);
 
     Key<Integer> key = key1 = key2 = Key.get(Integer.class);
     state1 = null;
@@ -224,7 +222,7 @@ public class WeakKeySetTest extends TestCase {
     // Now that the backing set is emptied, key1 is released.
     awaitClear(weakKey1Ref);
   }
-  
+
   public void testEviction_keyAndSourceOverlap_nonNull() {
     TestState state1 = new TestState();
     TestState state2 = new TestState();
@@ -239,19 +237,19 @@ public class WeakKeySetTest extends TestCase {
     // Same source so still only one value.
     assertInSet(set, key2, 1, source);
 
-    WeakReference<Key<Integer>> weakKey1Ref = new WeakReference<Key<Integer>>(key1);
-    WeakReference<Key<Integer>> weakKey2Ref = new WeakReference<Key<Integer>>(key2);
-    WeakReference<Object> weakSourceRef = new WeakReference<Object>(source);
+    WeakReference<Key<Integer>> weakKey1Ref = new WeakReference<>(key1);
+    WeakReference<Key<Integer>> weakKey2Ref = new WeakReference<>(key2);
+    WeakReference<Object> weakSourceRef = new WeakReference<>(source);
 
     Key<Integer> key = key1 = key2 = Key.get(Integer.class);
     state1 = null;
 
     awaitFullGc();
 
- // Same source so still only one value.
+    // Same source so still only one value.
     assertInSet(set, key, 1, source);
     assertInSet(set, key1, 1, source);
-    
+
     source = null;
 
     awaitFullGc();
@@ -291,12 +289,12 @@ public class WeakKeySetTest extends TestCase {
     set.add(key3, state3, source3);
     assertInSet(set, key1, 3, source1, source2, source3);
 
-    WeakReference<Key<Integer>> weakKey1Ref = new WeakReference<Key<Integer>>(key1);
-    WeakReference<Key<Integer>> weakKey2Ref = new WeakReference<Key<Integer>>(key2);
-    WeakReference<Key<Integer>> weakKey3Ref = new WeakReference<Key<Integer>>(key3);
-    WeakReference<Object> weakSource1Ref = new WeakReference<Object>(source1);
-    WeakReference<Object> weakSource2Ref = new WeakReference<Object>(source2);
-    WeakReference<Object> weakSource3Ref = new WeakReference<Object>(source3);
+    WeakReference<Key<Integer>> weakKey1Ref = new WeakReference<>(key1);
+    WeakReference<Key<Integer>> weakKey2Ref = new WeakReference<>(key2);
+    WeakReference<Key<Integer>> weakKey3Ref = new WeakReference<>(key3);
+    WeakReference<Object> weakSource1Ref = new WeakReference<>(source1);
+    WeakReference<Object> weakSource2Ref = new WeakReference<>(source2);
+    WeakReference<Object> weakSource3Ref = new WeakReference<>(source3);
 
     Key<Integer> key = key1 = key2 = key3 = Key.get(Integer.class);
     state1 = null;
@@ -316,7 +314,7 @@ public class WeakKeySetTest extends TestCase {
     assertInSet(set, key, 1, source3);
 
     awaitClear(weakKey2Ref);
-    
+
     source2 = null;
     awaitClear(weakSource2Ref);
     // Key1 will be referenced as the key in the sources backingSet and won't be
@@ -334,54 +332,69 @@ public class WeakKeySetTest extends TestCase {
   }
 
   public void testWeakKeySet_integration() {
-    Injector parentInjector = Guice.createInjector(new AbstractModule() {
-          @Override protected void configure() {
-            bind(Integer.class).toInstance(4);
-          }
-        });
+    Injector parentInjector =
+        Guice.createInjector(
+            new AbstractModule() {
+              @Override
+              protected void configure() {
+                bind(Integer.class).toInstance(4);
+              }
+            });
     assertNotBlacklisted(parentInjector, Key.get(String.class));
 
-    Injector childInjector = parentInjector.createChildInjector(new AbstractModule() {
-      @Override protected void configure() {
-        bind(String.class).toInstance("bar");
-      }
-    });
-    WeakReference<Injector> weakRef = new WeakReference<Injector>(childInjector);
+    Injector childInjector =
+        parentInjector.createChildInjector(
+            new AbstractModule() {
+              @Override
+              protected void configure() {
+                bind(String.class).toInstance("bar");
+              }
+            });
+    WeakReference<Injector> weakRef = new WeakReference<>(childInjector);
     assertBlacklisted(parentInjector, Key.get(String.class));
-    
+
     // Clear the ref, GC, and ensure that we are no longer blacklisting.
     childInjector = null;
     awaitClear(weakRef);
     assertNotBlacklisted(parentInjector, Key.get(String.class));
   }
-  
+
   public void testWeakKeySet_integration_multipleChildren() {
-    Injector parentInjector = Guice.createInjector(new AbstractModule() {
-          @Override protected void configure() {
-            bind(Integer.class).toInstance(4);
-          }
-        });
+    Injector parentInjector =
+        Guice.createInjector(
+            new AbstractModule() {
+              @Override
+              protected void configure() {
+                bind(Integer.class).toInstance(4);
+              }
+            });
     assertNotBlacklisted(parentInjector, Key.get(String.class));
     assertNotBlacklisted(parentInjector, Key.get(Long.class));
 
-    Injector childInjector1 = parentInjector.createChildInjector(new AbstractModule() {
-      @Override protected void configure() {
-        bind(String.class).toInstance("foo");
-      }
-    });
-    WeakReference<Injector> weakRef1 = new WeakReference<Injector>(childInjector1);
+    Injector childInjector1 =
+        parentInjector.createChildInjector(
+            new AbstractModule() {
+              @Override
+              protected void configure() {
+                bind(String.class).toInstance("foo");
+              }
+            });
+    WeakReference<Injector> weakRef1 = new WeakReference<>(childInjector1);
     assertBlacklisted(parentInjector, Key.get(String.class));
     assertNotBlacklisted(parentInjector, Key.get(Long.class));
-    
-    Injector childInjector2 = parentInjector.createChildInjector(new AbstractModule() {
-      @Override protected void configure() {
-        bind(Long.class).toInstance(6L);
-      }
-    });
-    WeakReference<Injector> weakRef2 = new WeakReference<Injector>(childInjector2);
+
+    Injector childInjector2 =
+        parentInjector.createChildInjector(
+            new AbstractModule() {
+              @Override
+              protected void configure() {
+                bind(Long.class).toInstance(6L);
+              }
+            });
+    WeakReference<Injector> weakRef2 = new WeakReference<>(childInjector2);
     assertBlacklisted(parentInjector, Key.get(String.class));
     assertBlacklisted(parentInjector, Key.get(Long.class));
-    
+
     // Clear ref1, GC, and ensure that we still blacklist.
     childInjector1 = null;
     awaitClear(weakRef1);
@@ -394,31 +407,40 @@ public class WeakKeySetTest extends TestCase {
     assertNotBlacklisted(parentInjector, Key.get(String.class));
     assertNotBlacklisted(parentInjector, Key.get(Long.class));
   }
-  
+
   public void testWeakKeySet_integration_multipleChildren_overlappingKeys() {
-    Injector parentInjector = Guice.createInjector(new AbstractModule() {
-          @Override protected void configure() {
-            bind(Integer.class).toInstance(4);
-          }
-        });
+    Injector parentInjector =
+        Guice.createInjector(
+            new AbstractModule() {
+              @Override
+              protected void configure() {
+                bind(Integer.class).toInstance(4);
+              }
+            });
     assertNotBlacklisted(parentInjector, Key.get(String.class));
 
-    Injector childInjector1 = parentInjector.createChildInjector(new AbstractModule() {
-      @Override protected void configure() {
-        bind(String.class).toInstance("foo");
-      }
-    });
-    WeakReference<Injector> weakRef1 = new WeakReference<Injector>(childInjector1);
+    Injector childInjector1 =
+        parentInjector.createChildInjector(
+            new AbstractModule() {
+              @Override
+              protected void configure() {
+                bind(String.class).toInstance("foo");
+              }
+            });
+    WeakReference<Injector> weakRef1 = new WeakReference<>(childInjector1);
     assertBlacklisted(parentInjector, Key.get(String.class));
-    
-    Injector childInjector2 = parentInjector.createChildInjector(new AbstractModule() {
-      @Override protected void configure() {
-        bind(String.class).toInstance("bar");
-      }
-    });
-    WeakReference<Injector> weakRef2 = new WeakReference<Injector>(childInjector2);
+
+    Injector childInjector2 =
+        parentInjector.createChildInjector(
+            new AbstractModule() {
+              @Override
+              protected void configure() {
+                bind(String.class).toInstance("bar");
+              }
+            });
+    WeakReference<Injector> weakRef2 = new WeakReference<>(childInjector2);
     assertBlacklisted(parentInjector, Key.get(String.class));
-    
+
     // Clear ref1, GC, and ensure that we still blacklist.
     childInjector1 = null;
     awaitClear(weakRef1);
@@ -431,88 +453,108 @@ public class WeakKeySetTest extends TestCase {
   }
 
   private static class TestState implements State {
+    @Override
     public State parent() {
       return new TestState();
     }
 
+    @Override
     public <T> BindingImpl<T> getExplicitBinding(Key<T> key) {
       return null;
     }
 
+    @Override
     public Map<Key<?>, Binding<?>> getExplicitBindingsThisLevel() {
       throw new UnsupportedOperationException();
     }
 
+    @Override
     public void putBinding(Key<?> key, BindingImpl<?> binding) {
       throw new UnsupportedOperationException();
     }
 
+    @Override
     public ScopeBinding getScopeBinding(Class<? extends Annotation> scopingAnnotation) {
       return null;
     }
 
+    @Override
     public void putScopeBinding(Class<? extends Annotation> annotationType, ScopeBinding scope) {
       throw new UnsupportedOperationException();
     }
 
+    @Override
     public void addConverter(TypeConverterBinding typeConverterBinding) {
       throw new UnsupportedOperationException();
     }
 
-    public TypeConverterBinding getConverter(String stringValue, TypeLiteral<?> type, Errors errors,
-        Object source) {
+    @Override
+    public TypeConverterBinding getConverter(
+        String stringValue, TypeLiteral<?> type, Errors errors, Object source) {
       throw new UnsupportedOperationException();
     }
 
+    @Override
     public Iterable<TypeConverterBinding> getConvertersThisLevel() {
       return ImmutableSet.of();
     }
 
     /*if[AOP]*/
+    @Override
     public void addMethodAspect(MethodAspect methodAspect) {
       throw new UnsupportedOperationException();
     }
 
+    @Override
     public ImmutableList<MethodAspect> getMethodAspects() {
       return ImmutableList.of();
     }
     /*end[AOP]*/
 
+    @Override
     public void addTypeListener(TypeListenerBinding typeListenerBinding) {
       throw new UnsupportedOperationException();
     }
 
+    @Override
     public List<TypeListenerBinding> getTypeListenerBindings() {
       return ImmutableList.of();
     }
 
+    @Override
     public void addProvisionListener(ProvisionListenerBinding provisionListenerBinding) {
       throw new UnsupportedOperationException();
     }
 
+    @Override
     public List<ProvisionListenerBinding> getProvisionListenerBindings() {
       return ImmutableList.of();
     }
 
+    @Override
     public void addScanner(ModuleAnnotatedMethodScannerBinding scanner) {
       throw new UnsupportedOperationException();
     }
 
+    @Override
     public List<ModuleAnnotatedMethodScannerBinding> getScannerBindings() {
       return ImmutableList.of();
     }
 
-    public void blacklist(Key<?> key, State state, Object source) {
-    }
+    @Override
+    public void blacklist(Key<?> key, State state, Object source) {}
 
+    @Override
     public boolean isBlacklisted(Key<?> key) {
       return true;
     }
 
+    @Override
     public Set<Object> getSourcesForBlacklistedKey(Key<?> key) {
       throw new UnsupportedOperationException();
     }
 
+    @Override
     public Object lock() {
       throw new UnsupportedOperationException();
     }
@@ -521,6 +563,7 @@ public class WeakKeySetTest extends TestCase {
       throw new UnsupportedOperationException();
     }
 
+    @Override
     public Map<Class<? extends Annotation>, Scope> getScopes() {
       return ImmutableMap.of();
     }
